@@ -2,7 +2,7 @@
 /******************************************************************************
  *  SiteBar 3 - The Bookmark Server for Personal and Team Use.                *
  *  Copyright (C) 2004  Gunnar Wrobel <sitebar@gunnarwrobel.de>               *
- *  Copyright (C) 2004  Ondrej Brablc <http://brablc.com/mailto?o>            *
+ *  Copyright (C) 2004-2006  Ondrej Brablc <http://brablc.com/mailto?o>       *
  *                                                                            *
  *  This program is free software; you can redistribute it and/or modify      *
  *  it under the terms of the GNU General Public License as published by      *
@@ -24,8 +24,6 @@ require_once('./inc/page.inc.php');
 require_once('./inc/errorhandler.inc.php');
 require_once('./inc/usermanager.inc.php');
 
-define ( "LOG_HTTP" , false);
-
 /* Information implemented
  *      HEAD    : The HTML Header of the page
  *      CHARSET : The character set of the page
@@ -36,8 +34,8 @@ define ( "LOG_HTTP" , false);
  *      FAVICON : The Favicon of the page
  *      ! If you wish to retrieve the favicon !
  *      ! from a normal webpage, do not       !
- *      ! specify "FAVICON" directly. Instead !
- *      ! only retrieve "FAVURL". It will be  !
+ *      ! specify 'FAVICON' directly. Instead !
+ *      ! only retrieve 'FAVURL'. It will be  !
  *      ! read from the header of the webpage !
  *      ! and will automatically retrieve and !
  *      ! check the favicon.                  !
@@ -57,7 +55,7 @@ define ('PP_ERR__FAVICON_TOO_BIG',      501); // Fatal error
 define ('PP_ERR__FAVICON_WRONG_SIZE',   502); // Fatal error
 define ('PP_ERR__FAVICON_WRONG_FORMAT', 503); // Fatal error
 
-class HTTPStream extends ErrorHandler
+class SB_HTTPStream extends SB_ErrorHandler
 {
     // Handle for the connection to the
     // page
@@ -72,7 +70,7 @@ class HTTPStream extends ErrorHandler
     // Port
     var $port;
 
-    function HTTPStream($timeout=5)
+    function SB_HTTPStream($timeout=5)
     {
         $this->timeout = $timeout;
     }
@@ -127,21 +125,21 @@ class HTTPStream extends ErrorHandler
 
     function put($data)
     {
-        if (LOG_HTTP) $this->log('> ', $data);
+        if (SB_LOG_HTTP) $this->log('> ', $data);
         return fputs($this->connection, $data);
     }
 
     function get($size=4096)
     {
         $data = fgets($this->connection, $size);
-        if (LOG_HTTP) $this->log('< ', $data);
+        if (SB_LOG_HTTP) $this->log('< ', $data);
         return $data;
     }
 
     function read($size=4096)
     {
         $data = fread($this->connection, $size);
-        if (LOG_HTTP) $this->log('< ', $data);
+        if (SB_LOG_HTTP) $this->log('< ', $data);
         return $data;
     }
 
@@ -160,11 +158,11 @@ class HTTPStream extends ErrorHandler
     }
 }
 
-class HTTPStream430 extends HTTPStream
+class SB_HTTPStream430 extends SB_HTTPStream
 {
-    function HTTPStream430($timeout=5)
+    function SB_HTTPStream430($timeout=5)
     {
-        $this->HTTPStream($timeout);
+        $this->SB_HTTPStream($timeout);
     }
 
     function setTimeOut($timeout=null)
@@ -179,8 +177,11 @@ class HTTPStream430 extends HTTPStream
 }
 
 
-class PageParser extends ErrorHandler
+class SB_PageParser extends SB_ErrorHandler
 {
+    var $um = null;
+    var $expertMode = false;
+
     // HTTP stream
     var $http = null;
 
@@ -208,21 +209,23 @@ class PageParser extends ErrorHandler
     // Information error messages
     var $errorCode = array();
 
-    function PageParser($url)
+    function SB_PageParser($url)
     {
-        $um =& UserManager::staticInstance();
-        $this->maxBytes = $um->getParam('config','max_icon_size');
+        $this->um =& SB_UserManager::staticInstance();
+
+        $this->expertMode = $this->um->getParam('user','expert_mode');
+        $this->maxBytes = $this->um->getParam('config','max_icon_size');
 
         /* This is the timeout while reading or writing data
          * The function name changed in PHP 4.3
          */
         if (version_compare(phpversion(), '4.3.0', '>='))
         {
-            $this->http = new HTTPStream430();
+            $this->http = new SB_HTTPStream430();
         }
         else
         {
-            $this->http = new HTTPStream();
+            $this->http = new SB_HTTPStream();
         }
 
         // Set the necessary path information
@@ -240,13 +243,13 @@ class PageParser extends ErrorHandler
         if (!$this->retrieveHTTPHeaders())
         {
             $this->isDead = true;
-            return;
+            return false;
         }
 
         // We wanted just validation
         if (!$info)
         {
-            return;
+            return true;
         }
 
         foreach ($info as $value)
@@ -258,7 +261,7 @@ class PageParser extends ErrorHandler
         // Get the page head so that we can get more information
         $this->retrieveHEADTag();
 
-        if (!$this->hasErrors())
+        if (!$this->hasErrors(E_ERROR))
         {
             foreach ($info as $value)
             {
@@ -278,17 +281,26 @@ class PageParser extends ErrorHandler
             if (!isset($this->info['FAVURL']))
             {
                 $defLoc = $this->base . '/favicon.ico';
-                $this->warn('Shortcut icon not found! Trying default favicon location %s.', array($defLoc));
+                if ($this->expertMode)
+                {
+                    $this->warn('Shortcut icon not found! Trying default favicon location %s.', array($defLoc));
+                }
                 $this->info['FAVURL'] = $defLoc;
             }
 
-            $this->setUrlInformation($this->info['FAVURL']);
+            if (!$this->setUrlInformation($this->info['FAVURL']))
+            {
+                return false;
+            }
 
             if (!$this->connect())
             {
                 unset($this->info['FAVURL']);
                 $this->errorCode['FAVURL'] = PP_ERR;
-                $this->warn('Favicon not found!');
+                if ($this->expertMode)
+                {
+                    $this->warn('Favicon not found!');
+                }
             }
             else
             {
@@ -303,10 +315,9 @@ class PageParser extends ErrorHandler
                     require_once('./inc/faviconcache.inc.php');
                     require_once('./inc/usermanager.inc.php');
 
-                    $um =& UserManager::staticInstance();
-                    $fc =& FaviconCache::staticInstance();
+                    $fc =& SB_FaviconCache::staticInstance();
 
-                    if ($um->getParam('config', 'use_favicon_cache')
+                    if ($this->um->getParam('config', 'use_favicon_cache')
                     && !$fc->isFaviconCached($this->info['FAVURL']))
                     {
                         // We have so save it
@@ -320,6 +331,31 @@ class PageParser extends ErrorHandler
                 }
             }
         }
+
+        return true;
+    }
+
+    function getContent()
+    {
+        if (SB_LOG_HTTP) $this->log("\rH:", "getContent\r");
+        if (!$this->connect())
+        {
+            return false;
+        }
+
+        $this->http->put('GET ' . $this->path . ' HTTP/1.0'."\r\n");
+        $this->http->put('Host: ' . $this->parsed['host'] . "\r\n");
+        $this->http->put('Accept: text/html'."\r\n");
+        $this->putCommonHeaders();
+
+        $lines = $this->readHTTPHeaders();
+        while ($line = $this->http->get())
+        {
+            $lines[] = trim($line);
+        }
+
+        $this->http->close();
+        return $lines;
     }
 
     function connect()
@@ -343,10 +379,10 @@ class PageParser extends ErrorHandler
         static $agent = null;
         if ($agent === null)
         {
-            $agent = "SiteBar/" . str_replace(' ', '', CURRENT_RELEASE);
+            $agent = 'SiteBar/' . str_replace(' ', '', SB_CURRENT_RELEASE) . ' (Bookmark Server; http://sitebar.org/)';
         }
 
-        $this->http->put( "User-Agent: $agent\r\n");
+        $this->http->put( 'User-Agent: '.$agent."\r\n");
 
         // We cannot use Keep-Alive if we do not want to complicate the communication a lot.
         // It is not guaranteed that the connection would be kept and we do usually only 3
@@ -354,9 +390,9 @@ class PageParser extends ErrorHandler
         // If someone wants to go this way, then he must count with other problems. He must
         // ensure that he flushes the stream, reads only what he should (some sites do not
         // sent Content-length!).
-        $this->http->put( "Connection: Close\r\n");
+        $this->http->put('Connection: Close'."\r\n");
 
-        $this->http->put( "Referer: " . Page::baseurl() . "/\r\n");
+        $this->http->put('Referer: ' . SB_Page::absBaseUrl() . "\r\n");
         if ($close)
         {
             $this->http->put( "\r\n");
@@ -368,26 +404,26 @@ class PageParser extends ErrorHandler
      * constructor.
      * The function has been adapted from the PEAR HTTP module.
      */
-    function retrieveHTTPHeaders($request = "HEAD")
+    function retrieveHTTPHeaders($request = 'HEAD')
     {
-        if (LOG_HTTP) $this->log("\nH:", "retrieveHTTPHeaders\n");
+        if (SB_LOG_HTTP) $this->log("\rH:", "retrieveHTTPHeaders\r");
         if (!$this->connect())
         {
             return false;
         }
 
-        $this->http->put( $request . " " . $this->path . " HTTP/1.0\r\n");
-        $this->http->put( "Host: " . $this->parsed['host'] . "\r\n");
+        $this->http->put($request . ' ' . $this->path . ' HTTP/1.0'."\r\n");
+        $this->http->put('Host: ' . $this->parsed['host'] . "\r\n");
         $this->putCommonHeaders();
 
         $this->info['HEADERS'] = $this->readHTTPHeaders();
 
         $status = $this->http->getMetaData();
-        if ($status['timed_out'] && $request == "HEAD")
+        if ($status['timed_out'] && $request == 'HEAD')
         {
             // Timeout. Might signify unsupported HEAD. Try a GET.
             $this->warn('HEAD unsupported, trying GET!'); // !! DEBUG
-            $this->retrieveHTTPHeaders("GET");
+            $this->retrieveHTTPHeaders('GET');
         }
         else if ($status['timed_out'])
         {
@@ -397,20 +433,23 @@ class PageParser extends ErrorHandler
         {
             $this->warn('Incomplete HTTP response!');
         }
-        else if ($this->info['HEADERS']['response_code'] == 405
-                 && $request == "HEAD")
+        else if (intval($this->info['HEADERS']['response_code']) == 405
+                 && $request == 'HEAD')
         {
-            $this->retrieveHTTPHeaders("GET");
+            $this->retrieveHTTPHeaders('GET');
         }
-        else if ($this->info['HEADERS']['response_code'] > 399)
+        else if (intval($this->info['HEADERS']['response_code']) > 399)
         {
             $this->warn('Page not found!');
         }
-        else if ($this->info['HEADERS']['response_code'] > 299)
+        else if (intval($this->info['HEADERS']['response_code']) > 299)
         {
             if ($this->redirects > $this->maxRedirects)
             {
-                $this->warn('No more than %s redirections allowed!', array($this->maxRedirects));
+                if ($this->expertMode)
+                {
+                    $this->warn('No more than %s redirections allowed!', array($this->maxRedirects));
+                }
             }
             else
             {
@@ -418,9 +457,16 @@ class PageParser extends ErrorHandler
 
                 $url = $this->mergeRedirect($this->parsed, $redirect);
 
-                $this->warn('Page redirected to %s!', array($url));
+                if ($this->expertMode)
+                {
+                    $this->warn('Page redirected to %s!', array($url));
+                }
+
+                if (!$this->setUrlInformation($url))
+                {
+                    return false;
+                }
 
-                $this->setUrlInformation($url);
                 $this->redirects++;
                 unset($this->info['HEADERS']);
 
@@ -434,15 +480,15 @@ class PageParser extends ErrorHandler
     // Retrieve the HTML HEAD portion of the page
     function retrieveHEADTag()
     {
-        if (LOG_HTTP) $this->log("\nH:", "retrieveHTTPHeaders\n");
+        if (SB_LOG_HTTP) $this->log("\rH:", "retrieveHEADTag\r");
         if (!$this->connect())
         {
             return;
         }
 
-        $this->http->put( "GET " . $this->path . " HTTP/1.0\r\n");
-        $this->http->put( "Host: " . $this->parsed['host'] . "\r\n");
-        $this->http->put( "Accept: text/html\r\n");
+        $this->http->put('GET ' . $this->path . ' HTTP/1.0'."\r\n");
+        $this->http->put('Host: ' . $this->parsed['host'] . "\r\n");
+        $this->http->put('Accept: text/html'."\r\n");
         $this->putCommonHeaders();
 
         $this->info['HEADERS'] = $this->readHTTPHeaders();
@@ -502,8 +548,8 @@ class PageParser extends ErrorHandler
                 {
                     if (preg_match ('/href=([\'"])(.+?)\1/i', $tag, $found))
                     {
-                        $favUrl = @parse_url($found[2]);
-                        $this->info['FAVURL'] = $this->mergeRedirect($this->parsed, $favUrl);
+                        $favurl = @parse_url($found[2]);
+                        $this->info['FAVURL'] = $this->mergeRedirect($this->parsed, $favurl);
                         return;
                     }
                 }
@@ -556,27 +602,67 @@ class PageParser extends ErrorHandler
         $this->errorCode['TITLE'] = PP_ERR;
     }
 
-    function retrieveFAVICON(&$ico)
+    function parseHEADTagForDESC()
     {
-        if (LOG_HTTP) $this->log("\nH:", "retrieveFAVICON\n");
-
-        if (!$this->connect())
+        if (preg_match('#<meta name=[\'"]Description[\'"]\s*content=[\'"](.*?)[\'"]#i', $this->info['HEAD'], $found))
         {
-            return PP_ERR;
+            $this->info['DESC'] = $found[1];
+            return;
         }
 
-        $this->http->put( "GET " . $this->path . " HTTP/1.0\r\n");
-        $this->http->put( "Host: " . $this->parsed['host'] . "\r\n");
-        $this->http->put( "Accept: image/*\r\n");
-        $this->putCommonHeaders();
+        $this->errorCode['DESC'] = PP_ERR;
+    }
 
-        $head = $this->readHTTPHeaders();
-
-        if (!isset($head['response_code']) || $head['response_code'] > 299)
+    function parseHEADTagForKEYWORDS()
+    {
+        if (preg_match('#<meta name=[\'"]Keywords[\'"]\s*content=[\'"](.*)[\'"]#i', $this->info['HEAD'], $found))
         {
-            $this->warn('Favicon not found!');
-            $this->http->close();
-            return PP_ERR;
+            $this->info['KEYWORDS'] = $found[1];
+            return;
+        }
+
+        $this->errorCode['KEYWORDS'] = PP_ERR;
+    }
+
+    function retrieveFAVICON(&$ico)
+    {
+        if (SB_LOG_HTTP) $this->log("\rH:", "retrieveFAVICON\r");
+
+        for ($i=0; $i<2; $i++)
+        {
+            if (!$this->connect())
+            {
+                return PP_ERR;
+            }
+
+            $this->http->put('GET ' . $this->path . ' HTTP/1.0'."\r\n");
+            $this->http->put('Host: ' . $this->parsed['host'] . "\r\n");
+            $this->http->put('Accept: image/*,*/*;q=0.5'."\r\n");
+            $this->putCommonHeaders();
+
+            $head = $this->readHTTPHeaders();
+
+            if (!isset($head['response_code']) || intval($head['response_code']) > 399)
+            {
+                if ($this->expertMode)
+                {
+                    $this->warn('Favicon not found!');
+                }
+                $this->http->close();
+                return PP_ERR;
+            }
+
+            if (!isset($head['response_code']) || intval($head['response_code']) > 299)
+            {
+                $this->http->close();
+                if (!$this->setUrlInformation($head['location']))
+                {
+                    return false;
+                }
+                continue;
+            }
+
+            break;
         }
 
         $ico = '';
@@ -628,7 +714,7 @@ class PageParser extends ErrorHandler
             {
                 // HTTP Headers are case insensitive
                 $header = strtolower(substr($line, 0, $pos));
-                $value  = strtolower(trim(substr($line, $pos + 1)));
+                $value  = trim(substr($line, $pos + 1));
                 $head[$header] = $value;
             }
         }
@@ -808,8 +894,8 @@ class PageParser extends ErrorHandler
          */
         if ( strlen($ico) >= 40
              && substr( $ico, 0, 2) == 'BM'
-             && PageParser::bigEndianBin2long( substr( $ico, 6, 4)) == 0
-             && PageParser::bigEndianBin2long( substr( $ico, 14, 4)) == 40)
+             && SB_PageParser::bigEndianBin2long( substr( $ico, 6, 4)) == 0
+             && SB_PageParser::bigEndianBin2long( substr( $ico, 14, 4)) == 40)
         {
             // Highly probable that this is really a BMP
             return true;
@@ -820,12 +906,12 @@ class PageParser extends ErrorHandler
          * (see http://www.daubnet.com/formats/ICO.html)
          */
         if ( strlen($ico) >= 4
-             && PageParser::bigEndianBin2short( substr( $ico, 0, 2)) == 0
-             && PageParser::bigEndianBin2short( substr( $ico, 2, 2)) == 1)
+             && SB_PageParser::bigEndianBin2short( substr( $ico, 0, 2)) == 0
+             && SB_PageParser::bigEndianBin2short( substr( $ico, 2, 2)) == 1)
         {
             // Check for the magic 40 after some header bytes
-            $offset = PageParser::bigEndianBin2short( substr( $ico, 4, 2)) * 16;
-            if (PageParser::bigEndianBin2long( substr( $ico, 6 + $offset, 4)) == 40)
+            $offset = SB_PageParser::bigEndianBin2short( substr( $ico, 4, 2)) * 16;
+            if (SB_PageParser::bigEndianBin2long( substr( $ico, 6 + $offset, 4)) == 40)
             {
                 // Highly probable that this is really an ICO
                 return true;
