@@ -4,7 +4,6 @@ $topic = array
 (
 '100' => 'Návod k použití',
 '101' => 'Chyť a upusť',
-'102' => 'Přidávání odkazů',
 '103' => 'Funkce nástrojové lišty',
 '200' => 'Příkazy',
 '210' => 'Správa seance',
@@ -15,9 +14,10 @@ $topic = array
 '260' => 'Operace se složkami',
 '270' => 'Operace s odkazy',
 '300' => 'Technické informace',
-'301' => 'Kompatibilita',
 '302' => 'Bezpečnostní mechanismus',
 '303' => 'Témata',
+'304' => 'Generátory výstupu',
+'305' => 'Stěhování',
 );
 
 ?>
