@@ -4,7 +4,6 @@ $topic = array
 (
 '100' => 'Hinweise zur Verwendung',
 '101' => 'Drag & Drop',
-'102' => 'Lesezeichen hinzufügen',
 '103' => 'Funktionen der Werkzeugleiste',
 '200' => 'Kommandos',
 '210' => 'Sessionverwaltung',
@@ -15,9 +14,10 @@ $topic = array
 '260' => 'Verzeichnisoperationen',
 '270' => 'Lesezeichen-Operationen',
 '300' => 'Technische Informationen',
-'301' => 'Kompatibilität',
 '302' => 'Sicherheitsmechanismus',
 '303' => 'Designs',
+'304' => 'Migration',
+'305' => 'Migration',
 );
 
 ?>
