@@ -4,7 +4,6 @@ $topic = array
 (
     100 => 'Usage Tips',
         101 => 'Drag &amp; Drop',
-        102 => 'Adding Links',
         103 => 'Tool Bar Functions',
 
     200 => 'Commands',
@@ -17,9 +16,10 @@ $topic = array
         270 => 'Link Operations',
 
     300 => 'Technical Info',
-        301 => 'Compatibility',
         302 => 'Security Mechanism',
         303 => 'Skins',
+        304 => 'Writers',
+        305 => 'Migration',
 );
 
 ?>
