@@ -1,7 +1,7 @@
 <?php
 /******************************************************************************
  *  SiteBar 3 - The Bookmark Server for Personal and Team Use.                *
- *  Copyright (C) 2004  Ondrej Brablc <http://brablc.com/mailto?o>            *
+ *  Copyright (C) 2004-2006  Ondrej Brablc <http://brablc.com/mailto?o>       *
  *                                                                            *
  *  This program is free software; you can redistribute it and/or modify      *
  *  it under the terms of the GNU General Public License as published by      *
@@ -18,8 +18,8 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
  ******************************************************************************/
 
-define ( "TRANSLATORS", "Translators");
-define ( "DEFAULT_LANGUAGE", "en_US");
+define ( 'TRANSLATORS', 'Translators');
+define ( 'DEFAULT_LANGUAGE', 'en_US');
 
 if (@!include_once('./inc/config.inc.php'))
 {
@@ -30,7 +30,7 @@ require_once('./inc/errorhandler.inc.php');
 require_once('./inc/page.inc.php');
 require_once('./inc/usermanager.inc.php');
 
-class Translator extends ErrorHandler
+class Translator extends SB_ErrorHandler
 {
     var $parts = array
     (
@@ -65,11 +65,26 @@ class Translator extends ErrorHandler
     );
 
     var $fmt = './locale/%s/%s';
+    var $infofmt = './locale/%s/%s';
     var $langs = array();
     var $gid = null;
+    var $plugin = '';
+    var $pluginCGI = '';
 
     function Translator()
     {
+        if (isset($_GET['plugin']) || isset($_POST['plugin']))
+        {
+            $plugin = isset($_GET['plugin'])?$_GET['plugin']:$_POST['plugin'];
+
+            if ($plugin != "" && preg_match('/^\w+$/', $plugin))
+            {
+                $this->dir = './plugins/'.$plugin;
+                $this->fmt = $this->dir.'/locale/%s/%s';
+                $this->pluginCGI = "plugin=".$plugin."&amp;";
+            }
+        }
+
         foreach ($this->parts as $part => $param)
         {
             include(sprintf($this->fmt,DEFAULT_LANGUAGE,$param['file']));
@@ -80,7 +95,7 @@ class Translator extends ErrorHandler
         {
             while (($lang = readdir($dir)) !== false)
             {
-                $filename = sprintf($this->fmt,$lang,"info.inc.php");
+                $filename = sprintf($this->infofmt,$lang,"info.inc.php");
                 if (!@file_exists($filename))
                 {
                     continue;
@@ -88,6 +103,7 @@ class Translator extends ErrorHandler
                 $stat = stat('./locale/'.$lang);
                 if ($stat[2] != 040777)
                 {
+                    echo "Wrong rights for $lang: " . $stat[2] . "<br>";
                     continue;
                 }
                 $this->langs[] = $lang;
@@ -152,15 +168,19 @@ class Translator extends ErrorHandler
     function addFile(&$zipfile, $lang, $file)
     {
         $file = sprintf($this->fmt, $lang, $file);
-        $handle = fopen ($file, "rb");
-        $contents = fread ($handle, filesize ($file));
-        fclose ($handle);
-        $zipfile->addFile($contents, $file);
+
+        if (is_file($file))
+        {
+            $handle = fopen ($file, 'rb');
+            $contents = fread ($handle, filesize ($file));
+            fclose ($handle);
+            $zipfile->addFile($contents, $file);
+        }
     }
 
     function check($needAdmin=false)
     {
-        $um = UserManager::staticInstance();
+        $um = SB_UserManager::staticInstance();
 
         // We must be logged in pass this point
         if (!$um->isLogged())
@@ -173,9 +193,9 @@ class Translator extends ErrorHandler
         {
             if (!$um->isAdmin())
             {
-                Page::head("Translator Error", "locale");
+                SB_Page::head('Translator Error', 'locale');
                 echo 'You must be admin to create new dir!';
-                Page::foot();
+                SB_Page::foot();
                 exit;
             }
             else
@@ -188,18 +208,18 @@ class Translator extends ErrorHandler
         $trGid = null;
         foreach ($groups as $gid => $rec)
         {
-            if ($rec["name"] == TRANSLATORS)
+            if ($rec['name'] == TRANSLATORS)
             {
                 $trGid = $gid;
                 break;
             }
         }
 
-        if (!$trGid)
+        if (!$um->isAdmin() && !$trGid)
         {
-            Page::head("Translator Error", "locale");
+            SB_Page::head('Translator Error', 'locale');
             echo 'This installation does not have group "' . TRANSLATORS . '"!';
-            Page::foot();
+            SB_Page::foot();
             exit;
         }
 
@@ -207,70 +227,111 @@ class Translator extends ErrorHandler
         $member = false;
         foreach ($myGroups as $gid => $rec)
         {
-            if ($rec["name"] == TRANSLATORS)
+            if ($rec['name'] == TRANSLATORS)
             {
                 $member = true;
                 break;
             }
         }
 
-        if (!$member)
+        if (!$um->isAdmin() && !$member)
         {
-            Page::head("Translator Error", "locale");
+            SB_Page::head('Translator Error', 'locale');
             echo 'You must be member of "' . TRANSLATORS . '" group to edit localizations! ';
             echo 'Ask <a href="command.php?command=Contact%20Moderator&amp;gid=' . $trGid . '">moderator</a> for membership.';
-            Page::foot();
+            SB_Page::foot();
             exit;
         }
     }
 
     function statistics()
     {
-        Page::head("Translator", "locale");
+        if (!defined("EMBEDDED"))
+        {
+            SB_Page::head('Translator', 'locale');
+            $server = defined("DOWNLOAD_SRV")?DOWNLOAD_SRV:"";
 ?>
 <h2>Translations</h2>
 
 <h4>Information for users</h4>
 <ul>
     <li>It is possible to download separate localizations or the whole
-        <a href='translator.php?download'>localization package</a> as ZIP files.
+        <a href='<?php echo $server?>translator.php?download'>localization package</a> as ZIP files.
 </ul>
 
 <h4>Information for translators</h4>
 <ul>
     <li>Although the languages can be edited on any SiteBar instance, the only authorized
         server to edit translation to be incorporated in the official release is
-        <a href="http://brablc.com/sitebar/translator.php">brablc.com</a>.
+        <a href="http://beta.sitebar.org/translator.php">beta.sitebar.org</a>.
     <li>In order to edit translations, you must have an account in this SiteBar installation
-        and be member of group "<?php echo TRANSLATORS?>".
+        and be either Administrator or member of group "<?php echo TRANSLATORS?>".
     <li>You can <a href='command.php?command=Sign%20Up'>create an account</a> yourself and ask
         moderator of the group to give you membership.
     <li>If there is some language missing then you have to
         <a href='command.php?command=Contact%20Admin'>contact admin</a> of this installation
         and ask him to <a href='translator.php?create'>create a directory</a> for your language, you will then be listed as the primary
         language translator.
-    <li><strong>This is real online translation.</strong> Open <a href="sitebar.php">Sitebar</a>,
+    <li><strong>This is real online translation.</strong> Open <a href="index.php">Sitebar</a>,
         change your language in the <a href='command.php?command=User%20Settings'>User Settings</a>
         and check the result.
     <li>Paragraphs that need update are marked with green color and &lt;@&gt; sign at the beginning.
-        Remove this sign after update.
+    There are 2 different cases: <br>
+    1. Source language contains a &lt;@&gt; The target language is dark green. This shows that we have changed something in the source language. Simply check if the translation needs to be updated. <br>
+    2. Target language contains a &lt;@&gt; The target language is light green. Check if the translation needs to be updated AND: Remove this sign after update.
 </ul>
 
-<h4>Differences<h4>
-<ul>
-    <li><a href="http://cvs.sourceforge.net/viewcvs.py/sitebar/sitebar3/locale/en_US/help.inc.php?r1=text&amp;tr1=1.5&amp;r2=1.7%3AHEAD&amp;tr2=&amp;diff_format=h">Help change from 3.2 to 3.2.5</a> - Validate Links
-</ul>
+<form method="get">
+Translate
+<select name='plugin' onChange="this.form.submit()">
+<option value=''>SiteBar</option>
+<?php
 
+            $dir = opendir('./plugins');
+
+            while (($plugin = readdir($dir)) !== false)
+            {
+                if ($plugin == '..')
+                {
+                    continue;
+                }
+
+                $plugdir = './plugins/'.$plugin;
+
+                if (!is_dir($plugdir))
+                {
+                    continue;
+                }
+
+                if (!is_dir($plugdir.'/locale'))
+                {
+                    continue;
+                }
+
+                echo "<option ". ($_GET['plugin']==$plugin?"selected":"") ." value='$plugin'>Plugin $plugin</option>\n";
+            }
+            closedir($dir);
+?>
+</select>
+</form>
+
+<p>
+
+<?php
+        }
+?>
 <table class='stat'>
 <tr>
     <th>Dir</th>
     <th>Language</th>
     <th>Author</th>
 <?php
+        $span = defined("EMBEDDED")?1:2;
+
         foreach ($this->parts as $part => $param)
         {
 ?>
-    <th colspan="2"><?php echo $this->parts[$part]['label']?></th>
+    <th colspan="<?php echo $span?>"><?php echo $this->parts[$part]['label']?></th>
 <?php
         }
 
@@ -280,31 +341,57 @@ class Translator extends ErrorHandler
 <?php
         foreach ($this->langs as $lang)
         {
-            include(sprintf($this->fmt,$lang,"info.inc.php"));
+            include(sprintf($this->infofmt,$lang,"info.inc.php"));
 
-            $author = $info['author'];
+            $authors = '';
+            $langs = $lang;
 
-            if ($info['url'])
+            if (isset($info['aliases']))
             {
-                $url = $info['url'];
+                $langs .= '<br>'. implode('<br>',$info['aliases']);
+            }
+
+            if (!isset($info['authors']))
+            {
+                $info['authors'][$info['author']] = $info['url'];
+            }
+
+            $count = 0;
+
+            foreach ($info['authors'] as $author => $url)
+            {
+                if ($count++)
+                {
+                    $authors .= '<br>';
+                }
+
                 if (strstr($url,"mailto:"))
                 {
                     $url =  str_replace("@", " ~at~ ", $url);
                     $url =  str_replace(".", " ~dot~ ", $url);
                 }
-                $author = "<a href='".$url."'>".$info['author']."</a>";
 
-                if (isset($info["co-authors"]))
+                if (strlen($url))
                 {
-                    $author .= '<br>' . $info["co-authors"];
+                    $authors .= "<a href='".$url."'>".$author."</a>";
                 }
+                else
+                {
+                    $authors .= $author;
+                }
+            }
+
+            if (isset($info['co-authors']))
+            {
+                $authors .= '<br>' . $info["co-authors"];
             }
 ?>
 <tr>
-    <td class='fixed'><?php echo $lang?>&nbsp;</td>
+    <td class="fixed"><?php echo $langs?>&nbsp;</td>
     <td><?php echo $info['language']?></td>
-    <td><?php echo $author?></td>
+    <td><?php echo $authors?></td>
 <?php
+
             foreach ($this->parts as $part => $param)
             {
                 eval('$'.$part.'=array();');
@@ -342,20 +429,31 @@ class Translator extends ErrorHandler
                     }
                 }
 
-                $ratio = sprintf( "%2.2f", $ok/$total*100);
+                $fmt = defined("EMBEDDED")?"%2d":"%2.2f";
+                $ratio = sprintf( $fmt, $ok/$total*100);
+
 ?>
     <td class='stat'><?php echo $ratio?>%</td>
+<?php
+                if (!defined("EMBEDDED"))
+                {
+?>
+
     <td class='statCmd'><?php
 
-                if ($lang!=DEFAULT_LANGUAGE)
-                {
-                ?>[<a href='translator.php?lang=<?php echo $lang?>&amp;edit=<?php echo $part?>'>EDIT</a>]<?php
-if ($missing) : ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=add&amp;edit=<?php echo $part?>'>ADD</a>]<?php endif;
-if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=upd&amp;edit=<?php echo $part?>'>UPD</a>]<?php endif;
+                    if ($lang!=DEFAULT_LANGUAGE)
+                    {
+                ?>[<a href='translator.php?lang=<?php echo $lang?>&amp;<?php echo $this->pluginCGI ?>edit=<?php echo $part?>'>EDIT</a>]<?php
+if ($missing) : ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;<?php echo $this->pluginCGI ?>cmd=add&amp;edit=<?php echo $part?>'>ADD</a>]<?php endif;
+if ($update && !$this->parts[$part]['inline']) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;<?php echo $this->pluginCGI ?>cmd=upd&amp;edit=<?php echo $part?>'>UPD</a>]<?php endif;
+                    }
                 }
             }
+
+        $server = defined("DOWNLOAD_SRV")?DOWNLOAD_SRV:"";
+
 ?>
-    <td class='stat'>[<a href='translator.php?download=<?php echo $lang?>'>Download</a>]</td>
+    <td class='stat'>[<a href='<?php echo $server?>translator.php?<?php echo $this->pluginCGI ?>download=<?php echo $lang?>'>Download</a>]</td>
 </tr>
 <?php
         }
@@ -363,38 +461,44 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 ?>
 </table>
 <?php
-        Page::foot();
+        if (!defined("EMBEDDED"))
+        {
+            SB_Page::foot();
+        }
     }
 
     function encodeValue($value, $show=false)
     {
         if ($show)
         {
-            $value = str_replace("&amp;#39;","'", htmlspecialchars($value));
+            $value = htmlspecialchars($value);
         }
         return $value;
     }
 
     function decodeValue(&$param, $value)
     {
-        if ($param['inline'])
-        {
-            return str_replace("'", "&#39;", stripslashes($value));
-        }
-        else
-        {
-            return str_replace("&#39;", "'", stripslashes($value));
-        }
+        return stripslashes($value);
     }
 
     function edit($lang, $part, $cmd, $save)
     {
-        Page::head("Edit Translation", "locale");
+        SB_Page::head('Edit Translation', 'locale');
 ?>
 <h2>Edit Translation</h2>
-[<a href="translator.php">Back to Translation List</a>]
+[<a href="translator.php?<?php echo $this->pluginCGI ?>">Back to Translation List</a>]
 <p>
 <?php
+        if (!isset($this->parts[$part]))
+        {
+            die("Unknown part in edit param!");
+        }
+
+        if (!preg_match('/^\w+$/',$lang))
+        {
+            die("Not allowed characters in lang param!");
+        }
+
         $param = $this->parts[$part];
         $file = sprintf($this->fmt,$lang,$param['file']);
 
@@ -419,21 +523,31 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
             $ok = 0;
             $total = 0;
 
-            if (!($fh = fopen($file,"wb")))
+            if (!$fh = fopen($file,'w'))
             {
-                echo "Cannot write results to file: $file" .
+                echo "Cannot write results to file: $file<br>\n";
+                echo "Sorry for inconvinience, if possible keep this page open and inform admin. When the problem is fixed you could just reload this page and post the data once again.<br>\n";
                 exit;
             }
 
             fwrite( $fh, "<?php\n\n");
 
-            if ($param['inline'])
+            $inline = $param['inline'];
+
+            if (!strlen($this->dir))
             {
-                fwrite( $fh, "\$$part = array\n(\n");
+                if ($inline)
+                {
+                    fwrite( $fh, "\$$part = array\n(\n");
+                }
+                else
+                {
+                    fwrite( $fh, "\$$part = array();\n\n");
+                }
             }
             else
             {
-                fwrite( $fh, "\$$part = array();\n\n");
+                $inline = false;
             }
 
             foreach ($default as $label => $value)
@@ -459,27 +573,36 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 
                 if (strlen($value))
                 {
-                    if ($param['inline'])
+                    if ($inline)
                     {
                         fwrite( $fh, sprintf("'%s' => '%s',\n", $label, $value));
                     }
                     else
                     {
                         $value = str_replace("\r\n","\n", $value);
-                        fwrite( $fh, "\$".$part."['".$label."'] = <<<_P\n");
+                        fwrite( $fh, "\$".$part."['".$label."'] = <<<_SBHD\n");
+
+                        // Do not allow here doc to be included in the string,
+                        // otherwise any php code would be executed.
+                        if (strstr($value,"_SBHD"))
+                        {
+                            die("Value must not contain _SBHD pattern!");
+                        }
+
                         fwrite( $fh, $value);
-                        fwrite( $fh, "\n_P;\n\n");
+                        fwrite( $fh, "\n_SBHD;\n\n");
                     }
                 }
             }
 
-            if ($param['inline'])
+            if ($inline)
             {
                 fwrite( $fh, ");\n\n");
             }
 
             fwrite( $fh, "?>\n");
             fclose( $fh);
+            chmod( $file, 0666);
 
             echo "<h4>Data saved to $file.</h4>\n";
 
@@ -491,25 +614,33 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 
 ?>
 <form method="POST">
-<table class='edit'>
+<table class="edit">
+<input type="hidden" name="dir" value="<?php $this->dir ?>">
 <?php
+
         $i  = 0;
 
         foreach ($default as $label => $value)
         {
-            $safeValue = isset($data[$label])?$data[$label]:'';
+            $SB_safeValue = isset($data[$label])?$data[$label]:'';
 
             $title  = $param['show_default']?$value:$label;
             $dCount = count(explode("\n", $title));
-            $lCount = count(explode("\n", $safeValue));
-            $rows   = max($lCount,$dCount);
+            $lCount = count(explode("\n", $SB_safeValue));
+            //this is why we calculate the rows of the input box by the length of the string
+            $rows = (strlen($SB_safeValue)/50);
+            if ($rows <1)
+            {
+                $rows =1;
+            }
 
-            if ($cmd=='add' && strlen($safeValue))
+
+            if ($cmd=='add' && strlen($SB_safeValue))
             {
                 continue;
             }
 
-            if ($cmd=='upd' && substr($safeValue,0,3)!='<@>')
+            if ($cmd=='upd' && substr($SB_safeValue,0,3)!='<@>') // && $param['inline']
             {
                 continue;
             }
@@ -535,20 +666,46 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
     <td class='label'><?php echo $this->encodeValue($title,true)?></td>
 </tr>
 <tr>
-    <td class='<?php echo strlen($safeValue)?(substr($safeValue,0,3)=='<@>'?'update':'value'):'missing'?>'>
+    <td class='<?php
+
+    if (strlen($SB_safeValue))
+    {
+        // this checks if the translation file OR the original language file starts with <@>
+        // - this way we need to update the english language only and start changed strings
+        // with a <@>
+        // changed by Olaf Noehring 2/2005
+        if (substr($SB_safeValue,0,3)=='<@>')
+        {
+            echo 'update';
+        }
+        elseif (substr($this->encodeValue($title,false),0,3)=='<@>')
+        {
+            echo 'updateFromOriginal';
+        }
+        else
+        {
+            echo 'value';
+        }
+    }
+    else
+    {
+        echo 'missing';
+    }
+
+    ?>'>
         <input type='hidden' name='label[<?php echo $i?>]' value='<?php echo $this->encodeValue($label)?>'>
-        <input type='hidden' name='md5[<?php echo $i?>]' value='<?php echo md5($safeValue)?>'>
+        <input type='hidden' name='md5[<?php echo $i?>]' value='<?php echo md5($SB_safeValue)?>'>
 <?php
             if ($param['inline'])
             {
 ?>
-        <input name='value[<?php echo $i?>]' value='<?php echo $this->encodeValue($safeValue)?>'>
+        <input name='value[<?php echo $i?>]' value='<?php echo $this->encodeValue($SB_safeValue)?>'>
 <?php
             }
             else
             {
 ?>
-        <textarea name='value[<?php echo $i?>]' rows='<?php echo $rows?>'><?php echo htmlspecialchars($safeValue)?></textarea>
+        <textarea name='value[<?php echo $i?>]' rows='<?php echo $rows?>'><?php echo htmlspecialchars($SB_safeValue)?></textarea>
 <?php
             }
 ?>
@@ -566,12 +723,15 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 <?php endif;?>
 </form>
 <?php
-        Page::foot();
+        if (!defined("EMBEDDED"))
+        {
+            SB_Page::foot();
+        }
     }
 
     function create()
     {
-        Page::head("Edit Translation", "locale");
+        SB_Page::head('Edit Translation', 'locale');
 ?>
 <h2>Create Translation</h2>
 [<a href="translator.php">Back to Translation List</a>]
@@ -581,7 +741,7 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
         {
             $ok = true;
 
-            foreach (array('dir','language','author','url') as $name)
+            foreach (array('newdir','language','author','url') as $name)
             {
                 if (!isset($_REQUEST[$name]) || !strlen($_REQUEST[$name]))
                 {
@@ -592,16 +752,16 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 
             if ($ok)
             {
-                $lang = $_REQUEST['dir'];
+                $lang = $_REQUEST['newdir'];
                 if (!mkdir('./locale/' . $lang))
                 {
-                    die("Cannot create directory!");
+                    die('Cannot create directory!');
                 }
                 chmod('./locale/' . $lang, 0777);
                 foreach ($this->parts as $part => $param)
                 {
                     $file = sprintf($this->fmt, $lang, $param['file']);
-                    if (!$fh = fopen($file, "w"))
+                    if (!$fh = fopen($file, 'w'))
                     {
                         die("Cannot create file $file!");
                     }
@@ -611,7 +771,7 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
                 }
 
                 $file = sprintf($this->fmt, $lang, "info.inc.php");
-                if (!$fh = fopen($file, "w"))
+                if (!$fh = fopen($file, 'w'))
                 {
                     die("Cannot create file $file!");
                 }
@@ -621,8 +781,10 @@ if ($update) :  ?><br>[<a href='translator.php?lang=<?php echo $lang?>&amp;cmd=u
 \$info = array
 (
     'language' => '${_REQUEST['language']}',
-    'author' => '${_REQUEST['author']}',
-    'url' => '${_REQUEST['url']}',
+    'authors' => array
+    (
+    '${_REQUEST['author']}' => '${_REQUEST['url']}',
+    ),
 );
 
 ?>
@@ -641,7 +803,7 @@ _INFO;
 <table>
 <tr>
     <th>Directory</th>
-    <td><input name='dir'></td>
+    <td><input name="newdir"></td>
 </tr>
 <tr>
     <th>Language</th>
@@ -661,7 +823,10 @@ _INFO;
 </form>
 <?php
         }
-        Page::foot();
+        if (!defined("EMBEDDED"))
+        {
+            SB_Page::foot();
+        }
     }
 }
 
